import json

from flask import Blueprint, jsonify, Response

from inventory_provider.routes import common

routes = Blueprint("msr-query-routes", __name__)


ACCESS_SERVICES_LIST_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema#",

    "definitions": {
        "service": {
            "type": "object",
            "properties": {
                "id": {"type": "integer"},
                "name": {"type": "string"},
                "equipment": {"type": "string"},
                "pop_name": {"type": "string"},
                "other_end_equipment": {"type": "string"},
                "other_end_pop_name": {"type": "string"},
                "speed_value": {"type": "integer"},
                "speed_unit": {"type": "string"}
            },
            "required": [
                "id", "name",
                "pop_name", "equipment",
                "other_end_pop_name", "other_end_equipment",
                "speed_value", "speed_unit"
            ],
            "additionalProperties": False
        }
    },

    "type": "array",
    "items": {"$ref": "#/definitions/service"}
}


@routes.after_request
def after_request(resp):
    return common.after_request(resp)


@routes.route("/access-services", methods=['GET', 'POST'])
@common.require_accepts_json
def access_services():
    """
    Handler for `/msr/access-services`.

    This method is in development, not yet used.

    The response will be formatted according to the following schema:

    .. asjson::
       inventory_provider.routes.msr.ACCESS_SERVICES_LIST_SCHEMA

    :return:
    """
    redis = common.get_current_redis()

    def _services():
        for k in redis.scan_iter('opsdb:access_services:*'):
            service = redis.get(k.decode('utf-8')).decode('utf-8')
            yield json.loads(service)

    cache_key = 'classifier-cache:msr:access-services'
    result = redis.get(cache_key)

    if result:
        result = json.loads(result.decode('utf-8'))
    else:
        result = list(_services())
        # cache this data for the next call
        redis.set(cache_key, json.dumps(result).encode('utf-8'))

    if not result:
        return Response(
            response='no access services found',
            status=404,
            mimetype="text/html")

    return jsonify(result)
